<?php

declare(strict_types = 1);

namespace Drupal\paragraphs_usage\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Entity\EntityFieldManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for Paragraph usage routes.
 */
class ParagraphsUsageController extends ControllerBase {

  /**
   * Entity Field Manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * ParagraphsUsageController constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManager $entity_field_manager
   *   Entity Field Manager service.
   */
  public function __construct(EntityFieldManager $entity_field_manager) {
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_field_manager = $container->get('entity_field.manager');
    return new static($entity_field_manager);
  }

  /**
   * Get Paragraphs usage.
   *
   * @param string $paragraphs_type
   *   Paragraphs type.
   *
   * @return array
   *   return render array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getUsage(string $paragraphs_type): array {
    $build = [];
    $header = [
      'label' => t('Label'),
      'machine_name' => t('Machine name'),
    ];
    if ($this->checkExistingParagraphs($paragraphs_type)) {
      $types = $this->entityTypeManager()
        ->getStorage('node_type')
        ->loadMultiple();
      foreach ($types as $type) {
        $base_fields = $this->entityFieldManager
          ->getFieldDefinitions('node', $type->id());
        foreach ($base_fields as $field_name => $definition) {
          if ($definition->getType() === 'entity_reference_revisions') {
            if (in_array($paragraphs_type, $definition->getSetting('handler_settings')['target_bundles'])) {
              $url = Url::fromRoute('entity.node_type.edit_form', ['node_type' => $type->id()]);
              $type_link = Link::fromTextAndUrl($type->label(), $url);

              $rows[] = [
                $type_link, $type->id(),
              ];
            }
          }
        }
      }
      if (empty($rows)) {
        return $build['empty'] = [
          '#type' => 'item',
          '#markup' => $this->t('This paragraph is not used in any content type.'),
        ];
      }
      return [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      ];
    }
  }

  /**
   * Check if paragraphs tupe exist.
   *
   * @param string $paragraphs_type
   *   Paragraphs type.
   *
   * @return bool
   *   Return TRUE if paragraphs type exist
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function checkExistingParagraphs(string $paragraphs_type): bool {
    $paragraph = $this->entityTypeManager()->getStorage('paragraphs_type')->load($paragraphs_type);
    if ($paragraph) {
      return TRUE;
    }
    throw new NotFoundHttpException();
  }

}
